public class MockIcon : MockItem
{
    public MockItemChoiceProperty image;

    public MockIcon ()
    {
        x = 0;
        y = 0;
        width = 20;
        height = 20;
        resizable = true;
        image = new MockItemChoiceProperty ("image", _("Image"), "cross-box");
        image.add_choice ("battery", _("Battery"));
        image.add_choice ("circle", _("Circle"));
        image.add_choice ("cog", _("Cog"));
        image.add_choice ("cross-box", _("Crossed Box"));
        image.add_choice ("power", _("Power"));
        image.add_choice ("search", _("Search"));
        image.add_choice ("speaker", _("Speaker"));
        image.add_choice ("spinner", _("Spinner"));
        image.add_choice ("star", _("Star"));
        image.add_choice ("tick", _("Tick"));
        image.add_choice ("triangle-up", _("Triangle (up)"));
        image.add_choice ("triangle-right", _("Triangle (right)"));
        image.add_choice ("triangle-down", _("Triangle (down)"));
        image.add_choice ("triangle-left", _("Triangle (left)"));
        image.add_choice ("wifi", _("WiFi"));
        image.value_changed.connect (() => { redraw (); });
        add_property (image);
    }

    public override MockItem copy ()
    {
        var copy = new MockIcon ();
        copy.x = x;
        copy.y = y;
        copy.width = width;
        copy.height = height;
        copy.image.value = image.value;

        return copy;
    }

    public override void draw (Cairo.Context c)
    {
        switch (image.value)
        {
        case "triangle-up":
        case "triangle": /* TEMP: Backwards compatibility for mockup <= 0.0.6 */
            draw_triangle_up (c);
            break;
        case "triangle-right":
            draw_triangle_right (c);
            break;
        case "triangle-down":
            draw_triangle_down (c);
            break;
        case "triangle-left":
            draw_triangle_left (c);
            break;
        case "box": /* TEMP: Backwards compatibility for mockup <= 0.0.6 */
            draw_box (c);
            break;
        default:
        case "cross-box":
            draw_cross_box (c);
            break;
        case "star":
            draw_star (c);
            break;
        case "tick":
            draw_tick (c);
            break;
        case "circle":
            draw_circle (c);
            break;
        case "spinner":
            draw_spinner (c);
            break;
        case "cog":
            draw_cog (c);
            break;
        case "search":
            draw_search (c);
            break;
        case "wifi":
            draw_wifi (c);
            break;
        case "battery":
            draw_battery (c);
            break;
        case "speaker":
            draw_speaker (c);
            break;
        case "power":
            draw_power (c);
            break;
        }
    }

    private void draw_box (Cairo.Context c)
    {
        c.rectangle (x + 0.5, y + 0.5, width, height);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_cross_box (Cairo.Context c)
    {
        var x_offset = (int) width * 0.1;
        var y_offset = (int) height * 0.1;
        var w = (int) (width * 0.8);
        var h = (int) (height * 0.8);
        c.rectangle (x + x_offset + 0.5, y + y_offset + 0.5, w, h);
        c.rel_line_to (w, h);
        c.rel_move_to (0, -h);
        c.rel_line_to (-w, h);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_triangle_up (Cairo.Context c)
    {
        var x_offset = (int) (width * 0.1);
        var y_offset = (int) (height * 0.1);
        var w = (int) (width * 0.8);
        var h = (int) (height * 0.8);
        c.move_to (x + x_offset + 0.5, y + height - y_offset + 0.5);
        c.rel_line_to (w * 0.5, -h);
        c.rel_line_to (w * 0.5, h);
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_triangle_right (Cairo.Context c)
    {
        var x_offset = (int) (width * 0.1);
        var y_offset = (int) (height * 0.1);
        var w = (int) (width * 0.8);
        var h = (int) (height * 0.8);
        c.move_to (x + x_offset + 0.5, y + y_offset + 0.5);
        c.rel_line_to (w, h * 0.5);
        c.rel_line_to (-w, h * 0.5);
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_triangle_down (Cairo.Context c)
    {
        var x_offset = (int) (width * 0.1);
        var y_offset = (int) (height * 0.1);
        var w = (int) (width * 0.8);
        var h = (int) (height * 0.8);
        c.move_to (x + x_offset + 0.5, y + y_offset + 0.5);
        c.rel_line_to (w * 0.5, h);
        c.rel_line_to (w * 0.5, -h);
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_triangle_left (Cairo.Context c)
    {
        var x_offset = (int) (width * 0.1);
        var y_offset = (int) (height * 0.1);
        var w = (int) (width * 0.8);
        var h = (int) (height * 0.8);
        c.move_to (x + x_offset + w + 0.5, y + y_offset + 0.5);
        c.rel_line_to (-w, h * 0.5);
        c.rel_line_to (w, h * 0.5);
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_star (Cairo.Context c)
    {
        var n_segments = 5;
        var radius = int.min (width, height) * 0.4;
        c.move_to (x + width / 2, y + height / 2 - radius);
        for (var i = 1; i < n_segments * 2; i++)
        {
            var px = Math.sin (2 * Math.PI * i / (n_segments * 2.0));
            var py = Math.cos (2 * Math.PI * i / (n_segments * 2.0));
            var r = radius;
            if (i % 2 == 1)
                r *= 0.5;
            c.line_to (x + width / 2 + px * r + 0.5, y + height / 2 - py * r + 0.5);
        }
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_tick (Cairo.Context c)
    {
        var x0 = (int) (width * 0.2);
        var y0 = (int) (height * 0.6);
        var x1 = (int) (width * 0.4);
        var y1 = (int) (height * 0.8);
        var x2 = (int) (width * 0.8);
        var y2 = (int) (height * 0.2);
        c.move_to (x + x0, y + y0);
        c.line_to (x + x1, y + y1);
        c.line_to (x + x2, y + y2);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_spinner (Cairo.Context c, int n_segments = 10)
    {
        var outer_radius = int.min (width, height) * 0.4;
        var inner_radius = outer_radius * 0.4;

        var a = Math.PI / n_segments;
        c.set_line_width (1.0);
        for (var i = 0; i < n_segments; i++)
        {
            var a0 = a * 2 * i;
            var a1 = a0 + a * 0.5;
            c.arc (x + width * 0.5, y + height * 0.5, outer_radius, a0, a1);
            c.arc_negative (x + width * 0.5, y + height * 0.5, inner_radius, a1, a0);
            c.close_path ();
            c.set_source_rgba (0.0, 0.0, 0.0, (i + 1.0) / n_segments);
            c.stroke ();
        }
    }

    private void draw_cog (Cairo.Context c, int n_segments = 8)
    {
        var radius = int.min (width, height) * 0.4;
        var a = 2 * Math.PI / (n_segments * 2.0);
        for (var i = 0; i < n_segments; i++)
        {
            var a0 = a * 2 * i;
            var a1 = a0 + a;
            var a2 = a0 + 2 * a;
            c.arc (x + width * 0.5 + 0.5, y + height * 0.5 + 0.5, radius, a0, a1);
            c.arc (x + width * 0.5 + 0.5, y + height * 0.5 + 0.5, radius * 0.8, a1, a2);
        }
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();

        c.arc (x + width * 0.5 + 0.5, y + height * 0.5 + 0.5, radius * 0.2, 0, 2 * Math.PI);
        c.stroke ();
    }

    private void draw_circle (Cairo.Context c)
    {
        var radius = int.min (width, height) * 0.4;
        c.arc (x + width / 2 + 0.5, y + height / 2 + 0.5, radius, 0, 2 * Math.PI);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_search (Cairo.Context c)
    {
        var radius = int.min (width, height) * 0.3;
        var offset = int.min (width, height) * 0.1;
        var cx = x + width / 2 - offset;
        var cy = y + height / 2 - offset;
        c.arc (cx, cy, radius, 0, 2 * Math.PI);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();

        c.move_to (cx + radius / Math.SQRT2, cy + radius / Math.SQRT2);
        c.rel_line_to (offset*2, offset*2);
        c.set_line_width (2.0);
        c.stroke ();
    }

    private void draw_wifi (Cairo.Context c)
    {
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        for (var i = 1; i < 4; i++)
        {
            c.arc (x + width / 2, y + height * 0.8, height * (i * 0.6 / 3), Math.PI * -0.7, Math.PI * -0.3);
            c.stroke ();
        }
        c.arc (x + width / 2, y + height * 0.8, 0.5, 0, 2 * Math.PI);
    }

    private void draw_battery (Cairo.Context c)
    {
        var h = (int) (height * 0.6);
        var h2 = (int) (height * 0.3);
        var w = width - int.min (5, (int) (height * 0.15));
        c.move_to (x + 0.5, y + (height - h) / 2 + 0.5);
        c.rel_line_to (w, 0);
        c.rel_line_to (0, (h - h2) / 2);
        c.rel_line_to (width - w, 0);
        c.rel_line_to (0, h2);
        c.rel_line_to (w - width, 0);
        c.rel_line_to (0, (h - h2) / 2);
        c.rel_line_to (-w, 0);
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_speaker (Cairo.Context c)
    {
        var w = (int) (width * 0.4);
        var h = (int) (height * 0.4);
        c.move_to (x + 0.5, y + height / 2 - h / 2 + 0.5);
        c.rel_line_to (w, 0);
        c.rel_line_to (w, -h / 2);
        c.rel_line_to (0, h * 2);
        c.rel_line_to (-w, -h / 2);
        c.rel_line_to (-w, 0);
        c.close_path ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();
    }

    private void draw_power (Cairo.Context c)
    {
        var radius = int.min (width, height) * 0.3;
        c.arc (x + width / 2, y + height / 2, radius, Math.PI * -0.3, Math.PI * 1.3);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();

        c.move_to (x + width / 2, y + height / 2 - radius * 0.25);
        c.rel_line_to (0, -radius);
        c.set_line_width (2.0);
        c.stroke ();
    }
}
