public class MockNote : MockItem
{
    public MockItemControlPoint target;
    public MockItemStringProperty text;

    public MockNote ()
    {
        x = 0;
        y = 0;
        width = 100;
        height = 60;
        resizable = true;
        target = new MockItemControlPoint ("target", 100, 100);
        target.moved.connect (() => { redraw (); });
        add_property (target);
        text = new MockItemStringProperty ("text", "Text", "NOTE");
        text.value_changed.connect (() => { redraw (); });
        add_property (text);
    }

    public override MockItem copy ()
    {
        var copy = new MockNote ();
        copy.x = x;
        copy.y = y;
        copy.width = width;
        copy.height = height;
        copy.target.x = target.x;
        copy.target.y = target.y;
        copy.text.value = text.value;

        return copy;
    }

    public override void draw (Cairo.Context c)
    {
        /* Line to target */
        c.move_to (x + width / 2, y + height / 2);
        c.line_to (target.x, target.y);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();

        /* Dot on target */
        c.arc (target.x, target.y, 4, 0, 2 * Math.PI);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.fill ();

        /* Text box */
        cairo_rounded_rectangle (c, x + 0.5, y + 0.5, width, height, 5);
        c.set_source_rgb (1.0, 1.0, 0.0);
        c.fill_preserve ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();

        /* Note text */
        c.move_to (x + 5, y + 15);
        c.text_path (text.value);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.fill ();
    }

    // FIXME: Also allow target to be selected
    //public override bool select (int x, int y)
}
