public class MockScrollBar : MockItem
{
    public MockItemIntegerProperty position;
    public MockItemIntegerProperty page_size;
    public MockItemBooleanProperty draw_border;

    public MockScrollBar ()
    {
        x = 0;
        y = 0;
        width = 100;
        height = 20;
        resizable = true;

        position = new MockItemIntegerProperty ("position", _("Position"), 60, 0, 100);
        position.value_changed.connect (() => { redraw (); });
        add_property (position);
        page_size = new MockItemIntegerProperty ("page-size", _("Page Size"), 60, 10, 100);
        page_size.value_changed.connect (() => { redraw (); });
        add_property (page_size);
        draw_border = new MockItemBooleanProperty ("draw-border", _("Draw Border"), true);
        draw_border.value_changed.connect (() => { redraw (); });
        add_property (draw_border);
    }

    public override MockItem copy ()
    {
        var copy = new MockScrollBar ();
        copy.x = x;
        copy.y = y;
        copy.width = width;
        copy.height = height;
        copy.position.value = position.value;
        copy.page_size.value = page_size.value;
        copy.draw_border.value = draw_border.value;

        return copy;
    }

    public override void draw (Cairo.Context c)
    {
        /* Draw slider */
        if (width >= height)
        {
            var p = (int) (width * (position.value / 100.0) * (100 - page_size.value) / 100.0);
            var w = (int) (width * page_size.value / 100.0);
            cairo_rounded_rectangle (c, x + p + 1, y + 1, w, height, 5);
        }
        else
        {
            var p = (int) (height * (position.value / 100.0) * (100 - page_size.value) / 100.0);
            var h = (int) (height * page_size.value / 100.0);
            cairo_rounded_rectangle (c, x + 1, y + p + 1, width, h, 5);
        }
        c.set_source_rgb (0.5, 0.5, 0.5);
        c.fill ();

        /* Draw border */
        if (draw_border.value)
        {
            cairo_rounded_rectangle (c, x + 0.5, y + 0.5, width, height, 5);
            c.set_source_rgb (0.0, 0.0, 0.0);
            c.set_line_width (1.0);
            c.stroke ();
        }
    }
}
