public class MockSlider : MockItem
{
    public MockItemIntegerProperty position;

    public MockSlider ()
    {
        x = 0;
        y = 0;
        width = 100;
        height = 20;
        resizable = true;

        position = new MockItemIntegerProperty ("position", _("Position"), 60, 0, 100);
        position.value_changed.connect (() => { redraw (); });
        add_property (position);
    }

    public override MockItem copy ()
    {
        var copy = new MockSlider ();
        copy.x = x;
        copy.y = y;
        copy.width = width;
        copy.height = height;

        return copy;
    }

    public override void draw (Cairo.Context c)
    {
        var p = position.value / 100.0;
        var slider_width = 6;

        /* Draw trough */
        if (width >= height)
        {
            c.move_to (x + slider_width / 2 + 0.5, y + height / 2 + 0.5);
            c.rel_line_to (width - slider_width, 0);
        }
        else
        {
            c.move_to (x + width / 2 + 0.5, y + slider_width / 2 + 0.5);
            c.rel_line_to (0, height - slider_width);
        }
        c.set_line_width (1.0);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.stroke ();

        /* Draw slider */
        if (width >= height)
        {
            var slider_x = (int) ((width - slider_width) * p);
            c.rectangle (x + slider_x + 0.5, y + 0.5, slider_width, height);
        }
        else
        {
            var slider_y = (int) ((height - slider_width) * (1.0 - p));
            c.rectangle (x + 0.5, y + slider_y + 0.5, width, slider_width);
        }
        c.set_source_rgb (1.0, 1.0, 1.0);
        c.fill_preserve ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.stroke ();
    }
}
