public class MockWindow : MockItem
{
    public MockItemStringProperty title;
    public MockItemBooleanProperty resize_property;

    public MockWindow ()
    {
        x = 0;
        y = 0;
        width = 240;
        height = 160;
        resizable = true;
        title = new MockItemStringProperty ("title", _("Title"), _("Window"));
        title.value_changed.connect (() => { redraw (); });
        add_property (title);
        resize_property = new MockItemBooleanProperty ("resizable", _("Resizable"), true);
        resize_property.value_changed.connect (() => { redraw (); });
        add_property (resize_property);
    }

    public override MockItem copy ()
    {
        var copy = new MockWindow ();
        copy.x = x;
        copy.y = y;
        copy.width = width;
        copy.height = height;
        copy.title.value = title.value;
        copy.resize_property.value = resize_property.value;

        return copy;
    }

    public override void draw (Cairo.Context c)
    {
        /* Draw window border */
        c.rectangle (x + 0.5, y + 0.5, width, height);
        c.set_source_rgb (1.0, 1.0, 1.0);
        c.fill_preserve ();
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.set_line_width (1.0);
        c.stroke ();

        /* Draw title bar */
        c.rectangle (x + 0.5, y + 0.5, width, 20);
        c.set_source_rgb (0.5, 0.5, 0.5);
        c.fill_preserve ();
        c.stroke ();

        var offset = 10;

        /* Close */
        c.arc (x + offset, y + 10, 7, 0, 2 * Math.PI);
        c.set_source_rgb (0.75, 0.0, 0.0);
        c.fill_preserve ();
        c.move_to (x + 10 - 3.5, y + 10 - 3.5);
        c.rel_line_to (7, 7);
        c.rel_move_to (0, -7);
        c.rel_line_to (-7, 7);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.stroke ();
        offset += 20;

        /* Minimize */
        c.arc (x + offset, y + 10, 7, 0, 2 * Math.PI);
        c.set_source_rgb (0.75, 0.75, 0.75);
        c.fill_preserve ();
        c.move_to (x + 30 - 3.5, y + 9.5);
        c.rel_line_to (7, 0);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.stroke ();
        offset += 20;

        /* Maximize */
        if (resize_property.value)
        {
            c.arc (x + offset, y + 10, 7, 0, 2 * Math.PI);
            c.set_source_rgb (0.75, 0.75, 0.75);
            c.fill_preserve ();
            c.rectangle (x + 50 - 3.5, y + 10 - 3.5, 7, 7);
            c.set_source_rgb (0.0, 0.0, 0.0);
            c.stroke ();
            offset += 20;
        }

        /* Draw title */
        c.move_to (x + offset, y + 15);
        c.text_path (title.value);
        c.set_source_rgb (0.0, 0.0, 0.0);
        c.fill ();
    }
}
