class MockupView : Gtk.Fixed
{
    private double select_x;
    private double select_y;
    private bool resizing = false;
    private bool dragging = false;
    private List<MockItem> template_items;
    private Gtk.Box? controls_box = null;

    private const int TRAY_WIDTH = 140;

    private Mockup? _mockup = null;
    public Mockup? mockup
    {
        get { return _mockup; }
        set
        {
            if (_mockup != null)
            {
                _mockup.foreach_item ((i) => { item_removed_cb (i); });
                SignalHandler.disconnect_by_func (_mockup, null, this);
            }

            _mockup = value;
            if (_mockup != null)
            {
                _mockup.item_added.connect (item_added_cb);
                _mockup.item_removed.connect (item_removed_cb);
                _mockup.selected_item_changed.connect (selected_item_changed_cb);
                _mockup.foreach_item ((i) => { item_added_cb (i); });
            }
            queue_draw ();
        }
    }

    public MockupView ()
    {
        set_has_window (true);
        add_events (Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON1_MOTION_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK);

        template_items = new List<MockItem> ();

        var y = 10;

        var window = new MockWindow ();
        window.x = 10;
        window.y = y;
        window.width = 120;
        window.height = 60;
        template_items.append (window);

        y += 70;

        var label = new MockLabel ();
        label.x = 10;
        label.y = y;
        label.width = 60;
        label.height = 20;
        template_items.append (label);
        
        var button = new MockButton ();
        button.x = 70;
        button.y = y;
        button.width = 60;
        button.height = 20;
        template_items.append (button);

        y += 30;

        var check = new MockCheck ();
        check.x = 10;
        check.y = y;
        template_items.append (check);

        var radio = new MockRadio ();
        radio.x = 40;
        radio.y = y;
        template_items.append (radio);

        var s = new MockSwitch ();
        s.x = 70;
        s.y = y;
        s.width = 60;
        s.height = 20;
        template_items.append (s);

        y += 30;

        var slider = new MockSlider ();
        slider.x = 10;
        slider.y = y;
        slider.width = 120;
        slider.height = 20;
        template_items.append (slider);

        y += 30;

        var progress_bar = new MockProgressBar ();
        progress_bar.x = 10;
        progress_bar.y = y;
        progress_bar.width = 120;
        progress_bar.height = 20;
        template_items.append (progress_bar);

        y += 30;

        var scroll_bar = new MockScrollBar ();
        scroll_bar.x = 10;
        scroll_bar.y = y;
        scroll_bar.width = 120;
        scroll_bar.height = 10;
        template_items.append (scroll_bar);

        y += 20;

        var icon = new MockIcon ();
        icon.x = 10;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "cross-box";
        template_items.append (icon);

        var box = new MockBox ();
        box.x = 40;
        box.y = y;
        box.width = 20;
        box.height = 20;
        template_items.append (box);

        icon = new MockIcon ();
        icon.x = 70;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "star";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 100;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "circle";
        template_items.append (icon);

        y += 30;

        icon = new MockIcon ();
        icon.x = 10;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "triangle-up";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 40;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "triangle-right";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 70;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "triangle-down";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 100;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "triangle-left";
        template_items.append (icon);

        y += 30;

        icon = new MockIcon ();
        icon.x = 10;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "cog";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 40;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "search";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 70;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "spinner";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 100;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "tick";
        template_items.append (icon);

        y += 30;

        icon = new MockIcon ();
        icon.x = 10;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "wifi";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 40;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "battery";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 70;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "speaker";
        template_items.append (icon);

        icon = new MockIcon ();
        icon.x = 100;
        icon.y = y;
        icon.width = 20;
        icon.height = 20;
        icon.image.value = "power";
        template_items.append (icon);

        y += 30;

        var note = new MockNote ();
        note.x = 10;
        note.y = y;
        note.width = 80;
        note.height = 20;
        note.target.x = 130;
        note.target.y = y + 10;
        template_items.append (note);
    }

    public override void get_preferred_width (out int minimum_width, out int natural_width)
    {
        minimum_width = 200;
        natural_width = 800;
    }

    public override void get_preferred_height (out int minimum_height, out int natural_height)
    {
        minimum_height = 200;
        natural_height = 400;
    }

    public override bool draw (Cairo.Context c)
    {
        /* Draw background */
        c.set_source (get_background_pattern (c));
        c.paint ();
        
        /* Draw item tray */
        c.rectangle (0, 0, TRAY_WIDTH, get_allocated_height ());
        c.set_source_rgba (0.5, 0.5, 0.5, 0.5);
        c.fill ();

        /* Draw template items */
        if (mockup.selected_item == null)
        {
            foreach (var item in template_items)
                item.draw (c);
        }

        /* Draw configure elements */
        base.draw (c);

        if (mockup == null)
            return true;

        c.translate (TRAY_WIDTH, 0);
        mockup.draw (c);

        /* Draw selection box */
        var item = mockup.selected_item;
        if (item != null)
        {
            c.rectangle (item.x + 0.5, item.y + 0.5, item.width, item.height);
            c.set_source_rgb (0.0, 0.0, 0.0);
            c.set_line_width (1.0);
            c.set_dash ({2.0, 3.0}, 0.5);
            c.stroke ();
            c.set_dash (null, 0.0);

            if (item.resizable)
            {
                c.rectangle (item.x + item.width - 3.5, item.y + item.height - 3.5, 7, 7);
                c.set_source_rgb (1.0, 1.0, 1.0);
                c.fill_preserve ();
                c.set_line_width (1.0);
                c.set_source_rgb (0.0, 0.0, 0.0);
                c.stroke ();
            }

            foreach (var prop in item.properties)
            {
                if (!(prop is MockItemControlPoint))
                    continue;

                var p = prop as MockItemControlPoint;
                c.arc (p.x, p.y, p.radius, 0, 2 * Math.PI);
                c.set_source_rgb (1.0, 1.0, 1.0);
                c.fill_preserve ();
                c.set_line_width (1.0);
                c.set_source_rgb (0.0, 0.0, 0.0);
                c.stroke ();
            }
        }

        return true;
    }

    public override void size_allocate (Gtk.Allocation allocation)
    {
        base.size_allocate (allocation);
        resize_controls ();
    }

    public override bool button_press_event (Gdk.EventButton event)
    {
        if (event.type != Gdk.EventType.BUTTON_PRESS)
            return false;

        var x = (int) event.x;
        var y = (int) event.y;
        
        if (mockup == null)
            return false;

        /* See if getting new item */
        if (mockup.selected_item == null)
        {
            foreach (var item in template_items)
            {
                if (item.select (x, y))
                {
                    var copy = item.copy ();
                    copy.x -= TRAY_WIDTH;
                    mockup.add_item (copy);
                    mockup.select_item (copy);
                    select_x = event.x;
                    select_y = event.y;
                    dragging = true;
                    resizing = false;
                    queue_draw ();
                    return true;
                }
            }
        }

        x -= TRAY_WIDTH;

        /* If a selected item see if resizing */
        // FIXME: Replace with control point
        if (mockup.selected_item != null && mockup.selected_item.resizable)
        {
            var item = mockup.selected_item;
            var handle_x = item.x + item.width - 3;
            var handle_y = item.y + item.height - 3;
            var dx = x - handle_x;
            var dy = y - handle_y;
            if (dx >= 0 && dx <= 7 && dy >= 0 && dy <= 7)
            {
                dragging = true;
                resizing = true;
                select_x = event.x;
                select_y = event.y;
                return true;
            }
        }
        resizing = false;

        MockItem item;
        MockItemControlPoint p;
        if (mockup.select (x, y, out item, out p))
        {
            mockup.select_item (item, p);
            select_x = event.x;
            select_y = event.y;
            dragging = true;
            queue_draw ();
            return true;
        }

        /* Selecting background deselects item */
        if (x > 0)
        {
            mockup.select_item (null);
            queue_draw ();
        }

        return false;
    }
    
    public override bool motion_notify_event (Gdk.EventMotion event)
    {
        if (!dragging || mockup.selected_item == null)
            return false;

        var dx = event.x - select_x;
        var dy = event.y - select_y;

        var x_step = (int) Math.trunc (dx / 10.0);
        var y_step = (int) Math.trunc (dy / 10.0);
        if (x_step == 0 && y_step == 0)
            return false;

        if (resizing)
            mockup.resize (x_step * 10, y_step * 10);
        else
            mockup.move (x_step * 10, y_step * 10);
        select_x += x_step * 10;
        select_y += y_step * 10;

        /* Don't allow to move above top of the screen */
        if (mockup.selected_item.y < 0)
        {
            select_y -= mockup.selected_item.y;
            mockup.selected_item.y = 0;
        }

        queue_draw ();

        return true;
    }

    public override bool button_release_event (Gdk.EventButton event)
    {
        if (!dragging || mockup.selected_item == null)
            return false;

        /* Dragging off the screen removes the item */
        if (mockup.selected_item.x < 0)
            mockup.remove_item (mockup.selected_item);

        dragging = false;
        return true;
    }

    private void item_added_cb (MockItem item)
    {
        item.redraw.connect (() => { queue_draw (); });
        queue_draw ();
    }

    private void item_removed_cb (MockItem item)
    {
        SignalHandler.disconnect_by_func (item, null, this);
        queue_draw ();
    }

    private void selected_item_changed_cb ()
    {
        /* Remove old grid */
        if (controls_box != null)
            controls_box.destroy ();
        controls_box = null;

        var item = mockup.selected_item;
        if (item == null)
            return;

        controls_box = new Gtk.Box (Gtk.Orientation.VERTICAL, 5);
        controls_box.visible = true;
        resize_controls ();
        put (controls_box, 0, 0);

        var add = new Gtk.Button.from_stock (Gtk.Stock.ADD);
        /* NOTE: This is a workaround for add.always_show_image = true which is only supported in Gtk+ 3.6 onwards */
        if (add.get_class ().find_property ("always-show-image") != null)
            add.set ("always-show-image", true);
        add.clicked.connect (() => { mockup.select_item (null); });
        controls_box.pack_start (add, false, false, 0);
        add.show ();        

        foreach (var prop in item.properties)
        {
            /* Ignore common properties */
            if (prop.id == "x" || prop.id == "y" || prop.id == "width" || prop.id == "height")
                continue;

            if (prop is MockItemStringProperty)
            {
                var p = prop as MockItemStringProperty;

                var label = new Gtk.Label ("%s:".printf (prop.description));
                label.set_alignment (0.0f, 0.5f);
                label.visible = true;
                controls_box.pack_start (label, false, false, 0);

                var entry = new Gtk.Entry ();
                entry.text = p.value;
                entry.visible = true;
                entry.changed.connect (() => { p.value = entry.text; });
                controls_box.pack_start (entry, false, false, 0);
            }
            else if (prop is MockItemBooleanProperty)
            {
                var p = prop as MockItemBooleanProperty;

                var label = new Gtk.Label ("%s:".printf (prop.description));
                label.set_alignment (0.0f, 0.5f);
                label.visible = true;
                controls_box.pack_start (label, false, false, 0);

                var s = new Gtk.Switch ();
                s.active = p.value;
                s.visible = true;
                s.halign = Gtk.Align.START;
                s.notify["active"].connect (() => { p.value = s.active; });
                controls_box.pack_start (s, false, false, 0);
            }
            else if (prop is MockItemIntegerProperty)
            {
                var p = prop as MockItemIntegerProperty;

                var label = new Gtk.Label ("%s:".printf (prop.description));
                label.set_alignment (0.0f, 0.5f);
                label.visible = true;
                controls_box.pack_start (label, false, false, 0);

                var adj = new Gtk.Adjustment (p.value, p.min, p.max, 1, 0, 0);
                var s = new Gtk.SpinButton (adj, 0, 0);
                s.visible = true;
                s.halign = Gtk.Align.START;
                adj.value_changed.connect (() => { p.value = (int) adj.value; });
                controls_box.pack_start (s, false, false, 0);
            }
            else if (prop is MockItemChoiceProperty)
            {
                var p = prop as MockItemChoiceProperty;

                var label = new Gtk.Label ("%s:".printf (prop.description));
                label.set_alignment (0.0f, 0.5f);
                label.visible = true;
                controls_box.pack_start (label, false, false, 0);

                var combo = new Gtk.ComboBox ();
                var model = new Gtk.ListStore (2, typeof (string), typeof (string));
                combo.model = model;
                var renderer = new Gtk.CellRendererText ();
                combo.pack_start (renderer, true);
                combo.add_attribute (renderer, "text", 1);
                foreach (var choice in p.choices)
                {
                    Gtk.TreeIter iter;
                    model.append (out iter);
                    model.set (iter, 0, choice.id, 1, choice.label);
                    if (choice.id == p.value)
                        combo.set_active_iter (iter);
                }
                combo.visible = true;
                combo.changed.connect (() =>
                {
                    Gtk.TreeIter iter;
                    if (!combo.get_active_iter (out iter))
                        return;

                    string id;
                    model.get (iter, 0, out id);
                    p.value = id;
                });
                controls_box.pack_start (combo, false, false, 0);
            }
            else if (prop is MockItemControlPoint)
            {
                /* Ignore control points - they are shown in the render */
            }
            else
            {
                warning ("Unknown property type");
            }
        }
    }

    private void resize_controls ()
    {
        if (controls_box != null)
        {
            var allocation = Gtk.Allocation ();
            allocation.x = 10;
            allocation.y = 10;
            allocation.width = 120;
            allocation.height = ((get_allocated_height () - 20) / 10) * 10;
            controls_box.size_allocate (allocation);
        }
    }

    private Cairo.Pattern? _background_pattern = null;
    private Cairo.Pattern get_background_pattern (Cairo.Context c)
    {
        if (_background_pattern != null)
            return _background_pattern;

        var surface = new Cairo.Surface.similar (c.get_target (), Cairo.Content.COLOR, 10, 10);
        var grid_c = new Cairo.Context (surface);
        grid_c.set_source_rgb (1.0, 1.0, 1.0);
        grid_c.paint ();

        grid_c.set_source_rgb (0.0, 0.0, 0.0);
        grid_c.rectangle (0.0, 0.0, 1.0, 1.0);
        grid_c.fill ();

        _background_pattern = new Cairo.Pattern.for_surface (surface);
        _background_pattern.set_extend (Cairo.Extend.REPEAT);

        return _background_pattern;
    }
}
